SystemOrganization addCategory: #'Gitocello-Core'!
SystemOrganization addCategory: #'Gitocello-GST'!
SystemOrganization addCategory: #'Gitocello-Morphic'!

Object subclass: #GCMapper
	instanceVariableNames: 'systemOrganizer repoDir files package'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gitocello-Core'!

!GCMapper commentStamp: '<historical>' prior: 0!
I am used to file out a package to a repository. I do not care at all what kind of repository. 
'repoDir' and 'packageName' are the only options that should be set on me.
You can use my 'files' field to find out which files I have filed out during my lifetime.

I just map packages and classes to files. Like so:
The packages and classes
		Package (Class1)
		Package-Core (Class2)
		Package-Base (Class3, Class4)
become
		Package/
			|- Core/
				|- Class2.st
			|- Base/
				|- Class3.st
				|- Class4.st
			|- Class1.st!

!GCMapper class methodsFor: 'accessing' stamp: ''!
extensionsFolder
	^'Extensions'
! !

!GCMapper class methodsFor: 'instance creation' stamp: ''!
newFor: aPackage
	^(self basicNew)
	    package: aPackage;
	    initialize;
	    yourself
! !

!GCMapper methodsFor: 'fileOut' stamp: ''!
createFolder: aSubfolder
	(File path: aSubfolder) exists 
	    ifFalse: [repoDir createDirectory: aSubfolder]
! !

!GCMapper methodsFor: 'fileOut' stamp: ''!
fileOutCategory: aCategory
	"This creates a sub-folder in the repository for the categories classes"
	| subfolder |
	subfolder := (aCategory asString findBetweenSubStrs: '-') last.
	(self package packageClasses 
	    intersection: (systemOrganizer superclassOrder: aCategory)) ifNotEmptyDo: 
		    [:arr | 
		    self createFolder: subfolder.
		    arr do: [:each | self fileOutClass: each in: (repoDir / subfolder) name]]
! !

!GCMapper methodsFor: 'fileOut' stamp: ''!
fileOutClass: aClass in: aPath
	"This files out the class in the according sub-folder"
	| stream path |
	stream := WriteStream on: (String new: 100).
	aClass 
	    fileOutOn: stream
	    moveSource: false
	    toFile: 0.
	path := aPath , Directory pathSeparator asString , aClass name.
	self writeSourceCodeFrom: stream to: path
! !

!GCMapper methodsFor: 'fileOut' stamp: ''!
fileOutExtensions: extensionMethods
	"For each extended class collect the extension Methods and file them out"
	| dict |
	self createFolder: self class extensionsFolder.
	dict := Dictionary new.
	extensionMethods do: 
		[:each | 
		dict at: each classSymbol ifAbsentPut: OrderedCollection new.
		(dict at: each classSymbol) add: each methodSymbol].
	dict keysDo: 
		[:key | 
		| stream file |
		stream := WriteStream on: (String new: 1000).
		file := (repoDir / 'Extensions') name , Directory pathSeparator asString 
			    , key asString.
		(dict at: key) do: 
			[:method | 
			(Smalltalk classNamed: key) 
			    printMethodChunk: method
			    withPreamble: true
			    on: stream
			    moveSource: false
			    toFile: 0].
		self writeSourceCodeFrom: stream to: file]
! !

!GCMapper methodsFor: 'fileOut' stamp: ''!
fileOutPackage
	"Files out the entire package into a folder structure"
	| packageInfo |
	packageInfo := PackageInfo named: self packageName.
	(packageInfo classes intersection: self package packageClasses) 
	    collect: [:cls | cls category]
	    thenDo: [:category | self fileOutCategory: category].
	(packageInfo extensionMethods 
	    select: [:m | self package packageClasses includes: m actualClass]) 
		ifNotEmptyDo: [:methods | self fileOutExtensions: methods]
! !

!GCMapper methodsFor: 'accessing' stamp: ''!
files
	^files
! !

!GCMapper methodsFor: 'initialize-release' stamp: ''!
initialize
	super initialize.
	files := Set new.
	systemOrganizer := Smalltalk organization
! !

!GCMapper methodsFor: 'accessing' stamp: ''!
package
	^package
! !

!GCMapper methodsFor: 'accessing' stamp: ''!
package: anObject
	package := anObject
! !

!GCMapper methodsFor: 'accessing' stamp: ''!
packageName
	^package packageName
! !

!GCMapper methodsFor: 'accessing' stamp: ''!
repoDir
	^repoDir
! !

!GCMapper methodsFor: 'accessing' stamp: ''!
repoDir: aFileDirectory
	aFileDirectory createDirectories.
	repoDir := aFileDirectory
! !

!GCMapper methodsFor: 'fileOut' stamp: ''!
writeSourceCodeFrom: aStream to: aFile
	"This writes the source code and adds to files"
	| converter f fileName |
	aStream contents isAsciiString 
	    ifTrue: [converter := MacRomanTextConverter new]
	    ifFalse: [converter := UTF8TextConverter new].
	fileName := aFile , ($. printString , ($s printString , $t printString)).
	f := MultiByteFileStream new open: fileName forWrite: true.
	f ifNil: [^self error: 'Cannot open file'].
	f lineEndConvention: #lf.
	(converter isMemberOf: UTF8TextConverter) 
	    ifTrue: 
		[f binary.
		UTF8TextConverter writeBOMOn: f].
	f text.
	f converter: converter.
	f nextPutAll: aStream contents.
	f close.
	files add: fileName
! !

Object subclass: #GCPackage
	instanceVariableNames: 'packageName packageClasses lastCommit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gitocello-Core'!

!GCPackage class methodsFor: 'instance creation' stamp: ''!
newFor: aPackage
	^(self new)
	    packageName: aPackage;
	    yourself
! !

!GCPackage methodsFor: 'accessing' stamp: ''!
classes
	"This converts, then sorts the class names, then finds the classes. Expensive, but run basically
	 only on intialization of the package and for the browser, which is slow anyway"
	| packageInfo |
	packageInfo := PackageInfo named: self packageName.
	^(((packageInfo classes asSet)
	    addAll: packageInfo extensionClasses;
	    yourself) collect: [:class | class name asString]) 
	    asSortedCollection collect: [:className | Smalltalk classNamed: className]
! !

!GCPackage methodsFor: 'accessing' stamp: ''!
lastCommit
	"Answer the value of lastCommit"
	^lastCommit
! !

!GCPackage methodsFor: 'accessing' stamp: ''!
lastCommit: anObject
	"Set the value of lastCommit"
	lastCommit := anObject
! !

!GCPackage methodsFor: 'accessing' stamp: ''!
packageClasses
	"Answer the value of packageClasses"
	packageClasses ifNil: [packageClasses := self classes].
	^packageClasses
! !

!GCPackage methodsFor: 'accessing' stamp: ''!
packageClasses: anObject
	"Set the value of packageClasses"
	packageClasses := anObject
! !

!GCPackage methodsFor: 'accessing' stamp: ''!
packageName
	"Answer the value of packageName"
	^packageName
! !

!GCPackage methodsFor: 'accessing' stamp: ''!
packageName: anObject
	"Set the value of packageName"
	packageName := anObject
! !

Object subclass: #GCRegistry
	instanceVariableNames: 'git'
	classVariableNames: ''
	poolDictionaries: 'Repositories RepositoryRoot'
	category: 'Gitocello-Core'!

GCRegistry class
	instanceVariableNames: 'instance'!

!GCRegistry commentStamp: '<historical>' prior: 0!
In my class I keep all the information on where to create/find git repositories in the file system (RepositoryRoot) and which Repositories have been created and how far they are along compared to their Monticello history.

My instance (of which there is only one) is used to create, remove and update repositories 
through the interface exposed in the 'commands' category.

I am well aware of the conversion to Gnu Smalltalk going on here, and I call the appropriate classes to do the conversion.!

GCRegistry class
	instanceVariableNames: 'instance'!

!GCRegistry class methodsFor: 'accessing' stamp: ''!
at: aPackage
	^self repositories at: aPackage
! !

!GCRegistry class methodsFor: 'class initialization' stamp: ''!
createDirectoryStructure: aString
	"Create the directory structure beneath the current image"
	^(FileDirectory default directoryNamed: aString)
		assureExistence.
! !

!GCRegistry class methodsFor: 'error handling' stamp: ''!
doesNotUnderstand: aMessage
	(self new respondsTo: aMessage selector) 
	    ifTrue: 
		[^self instance perform: aMessage selector withArguments: aMessage arguments].
	^super doesNotUnderstand: aMessage
! !

!GCRegistry class methodsFor: 'class initialization' stamp: ''!
initialize
	super initialize.
	Smalltalk addToStartUpList: self.
	self startUp.
	self instance: nil
! !

!GCRegistry class methodsFor: 'class variables' stamp: ''!
instance
	^instance
! !

!GCRegistry class methodsFor: 'class variables' stamp: ''!
instance: anObject
	instance := anObject
! !

!GCRegistry class methodsFor: 'class initialization' stamp: ''!
new
	"One global Registry is enough"
	^self instance ifNil: [instance := super new] ifNotNil: [:foo | instance]
! !

!GCRegistry class methodsFor: 'class variables' stamp: ''!
repositories
	Repositories ifNil: [Repositories := Dictionary new].
	^Repositories
! !

!GCRegistry class methodsFor: 'class variables' stamp: ''!
repositories: aDictionary
	Repositories := aDictionary
! !

!GCRegistry class methodsFor: 'class variables' stamp: ''!
repositoryRoot
	"Return the FileDirectory"
	^RepositoryRoot
! !

!GCRegistry class methodsFor: 'class variables' stamp: ''!
repositoryRoot: aStringOrDirectory
	"Set the FileDirectory either directly or from String"
	RepositoryRoot := aStringOrDirectory isString 
		    ifTrue: [self createDirectoryStructure: aStringOrDirectory]
		    ifFalse: [self createDirectoryStructure: aStringOrDirectory name]
! !

!GCRegistry class methodsFor: 'class initialization' stamp: ''!
startUp
	self 
	    repositoryRoot: 'package-cache' , FileDirectory slash , 'git-repositories'.
	self repositories keys copy do: 
		[:each | 
		(self repositoryRoot directoryNames includes: each) 
		    ifFalse: [Repositories removeKey: each]]

! !

!GCRegistry class methodsFor: 'class variables' stamp: ''!
tracksPackage: aPackageNameOrMCPackage
	^(self repositories keys includes: aPackageNameOrMCPackage) 
	    or: [self repositories keys includes: aPackageNameOrMCPackage name]
! !

!GCRegistry methodsFor: 'accessing' stamp: ''!
at: aPackage
	^self repositories at: aPackage
! !

!GCRegistry methodsFor: 'commands' stamp: ''!
commit: aPackage
	"This is part of the class interface - try not to break it"
	self
	    createFilesFor: aPackage;
	    run: #add
		in: aPackage asString
		with: #();
	    run: #commit:
		in: aPackage asString
		with: (Array with: (self commitMessagesFor: aPackage))
! !

!GCRegistry methodsFor: 'monticello-helpers' stamp: ''!
commitMessagesFor: aPackage
	"Creates a git-commit-msg from the non-commited Monticello versions"
	| cr messages |
	cr := Character lf asString.
	messages := WriteStream on: (String new: 100).
	((MCPackage named: aPackage) workingCopy ancestry breadthFirstAncestors 
	    copyUpTo: (self at: aPackage) lastCommit) do: 
		    [:next | 
		    messages 
			nextPutAll: cr , cr , next date asString , ': ' , next author 
				, ' from Squeak: ' , next message].
	(self at: aPackage) 
	    lastCommit: ((MCPackage named: aPackage) workingCopy ancestry ancestors 
		    ifEmpty: [nil]
		    ifNotEmpty: [:o | o first]).
	^self escapeForBash: messages contents
! !

!GCRegistry methodsFor: 'fileOut' stamp: ''!
createFilesFor: aPackage
	| files pkg |
	pkg := aPackage isString ifTrue: [self at: aPackage] ifFalse: [aPackage].
	files := (GCMapper newFor: pkg)
		    repoDir: RepositoryRoot / pkg packageName;
		    fileOutPackage;
		    files.
	files do: 
		[:file | 
		GCGstConvertCommand 
		    convert: file
		    from: 'squeak'
		    to: 'gst'].
	(GCGstPackageWriter newFor: pkg)
	    repoDir: RepositoryRoot / pkg packageName;
	    fileOutPackageXml
! !

!GCRegistry methodsFor: 'commands' stamp: ''!
createRepositoryFor: aPackage
	"This will do everything in its power to commit that package to git!!!!"
	(Repositories includesKey: aPackage) 
	    ifFalse: 
		[Repositories add: aPackage -> (GCPackage newFor: aPackage).
		self createFilesFor: aPackage.
		self run: #init in: aPackage asString].
	self commit: aPackage
! !

!GCRegistry methodsFor: 'commands' stamp: ''!
dumpImage
	"Dump all packages currently in the system"
	(SmalltalkImage current organization categories 
	    collect: [:c | c asString copyUpTo: $-]) asSet 
	    do: [:package | GCRegistry new createRepositoryFor: package]
! !

!GCRegistry methodsFor: 'monticello-helpers' stamp: ''!
escapeForBash: aString
	^(((aString replaceAll: Character cr with: Character lf) replaceAll: $!!
	    with: $.) replaceAll: $" with: $') 
	    replaceAll: $$
	    with: $S
! !

!GCRegistry methodsFor: 'initialize-release' stamp: ''!
initialize
	super initialize.
	git := GCGitWrapper new
! !

!GCRegistry methodsFor: 'commands' stamp: ''!
pull: aPackage
	"This might be removed at some time in the future"
	self run: #pull in: aPackage asString
! !

!GCRegistry methodsFor: 'commands' stamp: ''!
push: aPackage
	"This might be removed at some time in the future"
	self run: #push in: aPackage asString
! !

!GCRegistry methodsFor: 'commands' stamp: ''!
removeRepositoryFor: aPackage
	"This will not touch the filesystem, only our registry"
	Repositories removeKey: aPackage ifAbsent: []
! !

!GCRegistry methodsFor: 'accessing' stamp: ''!
repositories
	^Repositories
! !

!GCRegistry methodsFor: 'helpers' stamp: ''!
run: aGitCommand in: aPackage
	self 
	    run: aGitCommand
	    in: aPackage
	    with: (Array new: 0)
! !

!GCRegistry methodsFor: 'helpers' stamp: ''!
run: aGitCommand in: aPackage with: anArgumentsArray
	"Set the details we need and run"
	git
	    localPath: RepositoryRoot name , Directory pathSeparator asString 
			, aPackage;
	    perform: aGitCommand withArguments: anArgumentsArray
! !

!GCRegistry methodsFor: 'monticello-helpers' stamp: ''!
storeVersion: aWorkingCopy
	| package |
	package := aWorkingCopy package name.
	self
	    pull: package;
	    commit: package;
	    push: package
! !

SystemWindow subclass: #GCRepositoryBrowser
	instanceVariableNames: 'repositoryIndex selectedClassIndex classList selectedClasses classListMorph packageListMorph'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Gitocello-Morphic'!

!GCRepositoryBrowser class methodsFor: 'instance creation' stamp: ''!
open
	(self new)
	    createWindow;
	    openInWorld
! !

!GCRepositoryBrowser methodsFor: 'accessing-package-configuration' stamp: ''!
availableClasses
	classList ifNil: 
		[classList := GCRegistry repositories ifEmpty: [#()]
			    ifNotEmpty: [(GCRegistry at: self selectedRepository) classes]].
	^classList
! !

!GCRepositoryBrowser methodsFor: 'user interface' stamp: ''!
classListMorph
	^classListMorph ifNil: 
		[classListMorph := (PluggableListMorphOfMany 
			    on: self
			    list: #availableClasses
			    primarySelection: #selectedClassIndex
			    changePrimarySelection: #selectedClassIndex:
			    listSelection: #selectedClassesAt:
			    changeListSelection: #selectedClassesAt:put:
			    menu: nil)
			    color: self defaultBackgroundColor;
			    yourself]
! !

!GCRepositoryBrowser methodsFor: 'user interface' stamp: ''!
createWindow
	self
	    addMorph: self repositoryUrlField frame: (0 @ 0.01 corner: 0.8 @ 0.1);
	    addMorph: self syncButton frame: (0.8 @ 0.01 corner: 1.0 @ 0.1);
	    addMorph: self packageListMorph frame: (0 @ 0.1 corner: 0.5 @ 1.0);
	    addMorph: self classListMorph frame: (0.5 @ 0.1 corner: 1.0 @ 1.0);
	    setLabel: 'Package Git Repository Mirrors'
! !

!GCRepositoryBrowser methodsFor: 'accessing-package-configuration' stamp: ''!
currentlySelectedClasses
	^self availableClasses collect: [:cls | self packageClasses includes: cls]
! !

!GCRepositoryBrowser methodsFor: 'initialization' stamp: ''!
initialize
	self
	    selectedRepositoryIndex: 1;
	    selectedClassIndex: 1;
	    selectedClasses: self currentlySelectedClasses.
	super initialize
! !

!GCRepositoryBrowser methodsFor: 'accessing-package-configuration' stamp: ''!
packageClasses
	^GCRegistry repositories ifEmpty: [#()]
	    ifNotEmpty: [(GCRegistry at: self selectedRepository) packageClasses]
! !

!GCRepositoryBrowser methodsFor: 'user interface' stamp: ''!
packageListMorph
	^packageListMorph ifNil: 
		[packageListMorph := (PluggableListMorph 
			    on: self
			    list: #repositoryList
			    selected: #selectedRepositoryIndex
			    changeSelected: #selectedRepositoryIndex:)
			    color: self defaultBackgroundColor;
			    yourself]
! !

!GCRepositoryBrowser methodsFor: 'accessing' stamp: ''!
repositoryList
	^GCRegistry repositories keys asOrderedCollection
! !

!GCRepositoryBrowser methodsFor: 'accessing-registry-configuration' stamp: ''!
repositoryRootPath
	^GCRegistry repositoryRoot name
! !

!GCRepositoryBrowser methodsFor: 'accessing-registry-configuration' stamp: ''!
repositoryRootPath: aString
	^GCRegistry repositoryRoot: aString
! !

!GCRepositoryBrowser methodsFor: 'user interface' stamp: ''!
repositoryUrlField
	^(PluggableTextMorph 
	    on: self
	    text: #repositoryRootPath
	    accept: #repositoryRootPath:)
	    hideScrollBarsIndefinitely;
	    color: self defaultBackgroundColor;
	    yourself
! !

!GCRepositoryBrowser methodsFor: 'accessing-package-configuration' stamp: ''!
selectedClassIndex
	"Answer the value of selectedClassIndex"
	^selectedClassIndex
! !

!GCRepositoryBrowser methodsFor: 'accessing-package-configuration' stamp: ''!
selectedClassIndex: anObject
	"Set the value of selectedClassIndex"
	selectedClassIndex := anObject.
	self changed: #selectedClassIndex
! !

!GCRepositoryBrowser methodsFor: 'accessing-package-configuration' stamp: ''!
selectedClasses
	"Answer the value of selectedClasses"
	^selectedClasses
! !

!GCRepositoryBrowser methodsFor: 'accessing-package-configuration' stamp: ''!
selectedClasses: anObject
	"Set the value of selectedClasses"
	selectedClasses := anObject
! !

!GCRepositoryBrowser methodsFor: 'accessing-package-configuration' stamp: ''!
selectedClassesAt: anIndex
	^selectedClasses at: anIndex
! !

!GCRepositoryBrowser methodsFor: 'accessing-package-configuration' stamp: ''!
selectedClassesAt: anIndex put: aValue
	selectedClasses at: anIndex put: aValue = true.
	self
	    changed: #selectedClassIndex;
	    changed: #availableClasses;
	    updatePackageClasses
! !

!GCRepositoryBrowser methodsFor: 'accessing' stamp: ''!
selectedRepository
	^self repositoryList at: self selectedRepositoryIndex
! !

!GCRepositoryBrowser methodsFor: 'accessing' stamp: ''!
selectedRepositoryIndex
	"Answer the value of repositoryIndex"
	^repositoryIndex
! !

!GCRepositoryBrowser methodsFor: 'accessing' stamp: ''!
selectedRepositoryIndex: anObject
	"Set the value of repositoryIndex"
	repositoryIndex := anObject.
	classList := nil.
	self
	    selectedClassIndex: 1;
	    selectedClasses: self currentlySelectedClasses;
	    changed: #selectedRepositoryIndex;
	    changed: #selectedClassesAt:;
	    changed: #availableClasses
! !

!GCRepositoryBrowser methodsFor: 'user interface' stamp: ''!
syncButton
	^PluggableButtonMorph 
	    on: self
	    getState: #syncButtonOn
	    action: #syncButtonAction
	    label: #syncButtonString
! !

!GCRepositoryBrowser methodsFor: 'accessing-buttons' stamp: ''!
syncButtonAction
	GCRegistry createFilesFor: self selectedRepository
! !

!GCRepositoryBrowser methodsFor: 'accessing-buttons' stamp: ''!
syncButtonOn
	^GCRegistry repositories isEmpty not
! !

!GCRepositoryBrowser methodsFor: 'accessing-buttons' stamp: ''!
syncButtonString
	^'Sync repository'
! !

!GCRepositoryBrowser methodsFor: 'accessing' stamp: ''!
updatePackageClasses
	(GCRegistry at: self selectedRepository) 
	    packageClasses: (self availableClasses 
		    select: [:class | self selectedClasses at: (self availableClasses indexOf: class)])
! !

Object subclass: #GCGstPackageWriter
	instanceVariableNames: 'package packageXmlStream newline repoDir'
	classVariableNames: ''
	poolDictionaries: 'WritePackageXml'
	category: 'Gitocello-GST'!

!GCGstPackageWriter commentStamp: '<historical>' prior: 0!
I am very specific to GNU Smalltalk integration, as I write the package.xml file required to create GNU Smalltalk *.star format package files.
For now I honour dependencies as set in Monticello and simply file in all the classes in the package at once.
I will at some stage be able to properly handle test cases, which will make porting so much easier.!

!GCGstPackageWriter class methodsFor: 'instance creation' stamp: ''!
newFor: aPackage
	^(self new)
	    package: aPackage;
	    yourself
! !

!GCGstPackageWriter class methodsFor: 'preferences' stamp: ''!
writePackageXml
	<preference: 'Write Gnu Smalltalk package.xml files '
		category: 'Gitocello'
		description: ''
		type: #Boolean>
	^WritePackageXml ifNil: [WritePackageXml := false]
! !

!GCGstPackageWriter class methodsFor: 'preferences' stamp: ''!
writePackageXml: aBoolean
	WritePackageXml := aBoolean
! !

!GCGstPackageWriter methodsFor: 'stream-writing' stamp: ''!
classFilePathFor: aClass
	"Find the path to the classes fileOut"
	| subfolder |
	subfolder := (PackageInfo 
		    named: (aClass theNonMetaClass category asString findBetweenSubStrs: '-') 
			    first) 
			name = self packageName 
		    ifTrue: 
			[(aClass theNonMetaClass category asString findBetweenSubStrs: '-') last]
		    ifFalse: [GCMapper extensionsFolder].
	^subfolder , Directory pathSeparator asString , aClass name asString 
	    , $. printString , ($s printString , $t printString)
! !

!GCGstPackageWriter methodsFor: 'stream-writing' stamp: ''!
classes
	^(GCRegistry at: self packageName) packageClasses
! !

!GCGstPackageWriter methodsFor: 'stream-writing' stamp: ''!
createPackageXml
	self packageXmlStream reset.
	self
	    preamble;
	    requires;
	    tests;
	    fileIns;
	    files.
	self packageXmlStream nextPutAll: self postfix
! !

!GCGstPackageWriter methodsFor: 'stream-writing' stamp: ''!
fileIn: aClass
	self putAttribute: 'filein' with: (self classFilePathFor: aClass)
! !

!GCGstPackageWriter methodsFor: 'stream-writing' stamp: ''!
fileIns
	self classes reject: [:class | class allSuperclasses includes: TestCase]
	    thenDo: [:class | self fileIn: class]
! !

!GCGstPackageWriter methodsFor: 'fileOut' stamp: ''!
fileOutPackageXml
	| fileName converter file |
	self class writePackageXml 
	    ifTrue: 
		[self createPackageXml.
		self packageXmlStream contents isAsciiString 
		    ifTrue: [converter := MacRomanTextConverter new]
		    ifFalse: [converter := UTF8TextConverter new].
		fileName := self repoDir , Directory pathSeparator asString 
			    , 'package.xml'.
		File remove: fileName.
		file := MultiByteFileStream new open: fileName forWrite: true.
		file ifNil: [self error: 'Cannot open file'].
		file
		    text;
		    converter: converter;
		    nextPutAll: packageXmlStream contents;
		    close]
! !

!GCGstPackageWriter methodsFor: 'stream-writing' stamp: ''!
files
	self packageClasses 
	    do: [:class | self putAttribute: 'file' with: (self classFilePathFor: class)]
! !

!GCGstPackageWriter methodsFor: 'initialize-release' stamp: ''!
initialize
	super initialize.
	newline := Character lf.	"This should be configurable"
	self packageXmlStream: (WriteStream on: (String new: 100))
! !

!GCGstPackageWriter methodsFor: 'accessing' stamp: ''!
newline
	"Answer the value of newline"
	^newline
! !

!GCGstPackageWriter methodsFor: 'accessing' stamp: ''!
newline: anObject
	"Set the value of newline"
	newline := anObject
! !

!GCGstPackageWriter methodsFor: 'accessing' stamp: ''!
package
	"Answer the value of packageName"
	^package
! !

!GCGstPackageWriter methodsFor: 'accessing' stamp: ''!
package: anObject
	"Set the value of packageName"
	package := anObject
! !

!GCGstPackageWriter methodsFor: 'accessing' stamp: ''!
packageClasses
	"Answer the value of packageName"
	^package packageClasses
! !

!GCGstPackageWriter methodsFor: 'accessing' stamp: ''!
packageName
	^self package packageName
! !

!GCGstPackageWriter methodsFor: 'accessing' stamp: ''!
packageXmlStream
	"Answer the value of packageXmlStream"
	^packageXmlStream
! !

!GCGstPackageWriter methodsFor: 'accessing' stamp: ''!
packageXmlStream: anObject
	"Set the value of packageXmlStream"
	packageXmlStream := anObject
! !

!GCGstPackageWriter methodsFor: 'literals' stamp: ''!
postfix
	^'</package>' , Character cr asString
! !

!GCGstPackageWriter methodsFor: 'stream-writing' stamp: ''!
preamble
	{'<package>'.
	'<!!-- THIS FILE IS GENERATED!! EDITS _WILL_ BE LOST ON UPDATE!! -->'.
	''} 
		do: [:s | self packageXmlStream nextPutAll: s]
		separatedBy: [self packageXmlStream nextPut: newline].
	self
	    putAttribute: 'name' with: self packageName;
	    putAttribute: 'namespace' with: self packageName
! !

!GCGstPackageWriter methodsFor: 'stream-writing' stamp: ''!
putAttribute: anXmlAttribute with: someContent
	(self packageXmlStream)
	    nextPutAll: '<' , anXmlAttribute , '>';
	    nextPutAll: someContent;
	    nextPutAll: '</' , anXmlAttribute , '>';
	    nextPut: newline
! !

!GCGstPackageWriter methodsFor: 'accessing' stamp: ''!
repoDir
	"Answer the value of repoDir"
	^repoDir
! !

!GCGstPackageWriter methodsFor: 'accessing' stamp: ''!
repoDir: aDirectory
	"Set the value of repoDir"
	aDirectory isString 
	    ifTrue: [repoDir := aDirectory]
	    ifFalse: [repoDir := aDirectory name]
! !

!GCGstPackageWriter methodsFor: 'stream-writing' stamp: ''!
require: aPackage
	(self packageXmlStream)
	    nextPutAll: '<prereq>';
	    nextPutAll: aPackage;
	    nextPutAll: '</prereq>';
	    nextPut: newline
! !

!GCGstPackageWriter methodsFor: 'information-retrieval' stamp: ''!
requiredPackages
	^(MCPackage named: self packageName) workingCopy requiredPackages
! !

!GCGstPackageWriter methodsFor: 'stream-writing' stamp: ''!
requires
	self requiredPackages do: [:pkg | self require: pkg]
! !

!GCGstPackageWriter methodsFor: 'stream-writing' stamp: ''!
tests
	"Prepare the tests for usage with Gnu Smalltalk"
	| testClasses testCases |
	testClasses := self classes 
		    select: [:class | class allSuperclasses includes: TestCase].
	testCases := testClasses inject: String new
		    into: [:s :class | s , self packageName , '.' , class name asString , newline asString].
	self packageXmlStream nextPutAll: '<test>' , newline asString.
	testClasses do: [:class | self fileIn: class].
	self putAttribute: 'sunit' with: testCases.
	self packageXmlStream nextPutAll: '</test>' , newline asString
! !

!MCWorkingCopyBrowser methodsFor: '*gitocello-actions-override' stamp: ''!
saveVersion
	| repo |
	self canSave ifFalse: [^self].
	self checkForNewerVersions ifFalse: [^self].
	repo := self repository.
	workingCopy newVersion ifNotNil: 
		[:v | 
		(MCVersionInspector new version: v) show.
		Cursor wait showWhile: [repo storeVersion: v].
		MCCacheRepository default cacheAllFileNamesDuring: 
			[repo cacheAllFileNamesDuring: 
				[v allAvailableDependenciesDo: 
					[:dep | 
					(repo includesVersionNamed: dep info name) 
					    ifFalse: [repo storeVersion: dep]]]].
		(GCRegistry tracksPackage: workingCopy package) 
		    ifTrue: [GCRegistry storeVersion: workingCopy]]
! !

!MCWorkingCopyBrowser methodsFor: '*gitocello' stamp: ''!
trackWithGit
	GCRegistry createRepositoryFor: workingCopy package name
! !

!MCWorkingCopyBrowser methodsFor: '*gitocello' stamp: ''!
untrackWithGit
	GCRegistry removeRepositoryFor: workingCopy package name
! !

!MCWorkingCopyBrowser methodsFor: '*gitocello-morphic ui-override' stamp: ''!
workingCopyListMenu: aMenu
	workingCopy ifNil: [^aMenu].
	self fillMenu: aMenu
	    fromSpecs: #(#('add required package' #addRequiredPackage) #('clear required packages' #clearRequiredPackages) #('browse package' #browseWorkingCopy) #('view changes' #viewChanges) #('view history' #viewHistory) #('recompile package' #recompilePackage) #('revert package...' #revertPackage) #('unload package' #unloadPackage) #('delete working copy' #deleteWorkingCopy)).
	(Smalltalk includesKey: #SARMCPackageDumper) 
	    ifTrue: 
		[aMenu 
		    add: 'make SAR'
		    target: self
		    selector: #fileOutAsSAR].
	(GCRegistry tracksPackage: workingCopy package) 
	    ifFalse: 
		[aMenu 
		    add: 'track with Git'
		    target: self
		    selector: #trackWithGit]
	    ifTrue: 
		[aMenu 
		    add: 'untrack with Git'
		    target: self
		    selector: #untrackWithGit].
	^aMenu
! !

Object subclass: #GCGstConvertCommand
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: 'GstConvertBinaryPath RunGstConvert'
	category: 'Gitocello-GST'!

!GCGstConvertCommand commentStamp: '<historical>' prior: 0!
I am used for converting a Squeak source file to Gnu Smalltalk syntax using the gst-convert tool with the options found at http://smalltalk.gnu.org/faq/169!

!GCGstConvertCommand class methodsFor: 'private' stamp: ''!
allMethodsIn: aCategory
	^((self class methodDictionary asSet 
	    select: [:msg | msg methodCategory asString = aCategory]) 
		collect: [:each | each selector]) asArray
! !

!GCGstConvertCommand class methodsFor: 'callouts' stamp: ''!
basicConvert: aFilepath from: input to: output
	"see http://smalltalk.gnu.org/faq/169 for info on gst-convert command-line"
	| stream filename |
	stream := WriteStream on: (String new: 1000).
	stream nextPutAll: self gstConvertBinaryPath , ' -f ' , input , ' '.	"-F ', output, ' '."
	(self allMethodsIn: input , '-to-' , output , '-rules') 
	    do: [:method | self writeRules: (self perform: method) to: stream].
	stream nextPutAll: ' "' , aFilepath , '" "' , aFilepath , '.converted"'.
	GCCallout callout: stream contents.
	filename := FilePath stripPathFrom: aFilepath.
	File remove: aFilepath.
	(FileDirectory forFileName: aFilepath) rename: filename , '.converted'
	    toBe: filename
! !

!GCGstConvertCommand class methodsFor: 'bidirectional-rules' stamp: ''!
classNames
	^
	{'Float' -> 'FloatD'.
	'BlockContext' -> 'BlockClosure'.
	'MessageSend' -> 'DirectedMessage'.
	'DateAndTime' -> 'DateTime'.
	'UnhandledError' -> 'SystemExceptions.UnhandledException'}
! !

!GCGstConvertCommand class methodsFor: 'actions' stamp: ''!
convert: aFilepath from: input to: output
	self runGstConvert 
	    ifTrue: 
		[self 
		    basicConvert: aFilepath
		    from: input
		    to: output]
! !

!GCGstConvertCommand class methodsFor: 'callouts' stamp: ''!
flip: aString with: anotherString on: aFilepath
	| filename |
	(GCCallout 
	    callout: 'tr ' , aString , ' ' , anotherString , ' < "' , aFilepath 
		    , '" > "' , aFilepath 
		    , '.converted"') 
		= 0 
		ifTrue: 
		    [filename := FilePath stripPathFrom: aFilepath.
		    File remove: aFilepath.
		    (FileDirectory forFileName: aFilepath) rename: filename , '.converted'
			toBe: filename]
! !

!GCGstConvertCommand class methodsFor: 'gst-to-squeak-rules' stamp: ''!
gstBidirectional
	"Reverse all bidirectional rules to apply them for squeak -> gst conversion"
	^(self allMethodsIn: 'bidirectional-rules') inject: OrderedCollection new
	    into: 
		[:list :method | 
		(list asSet)
		    addAll: ((self perform: method) collect: [:each | each value -> each key]);
		    yourself]
! !

!GCGstConvertCommand class methodsFor: 'preferences' stamp: ''!
gstConvertBinaryPath
	<preference: 'Path to gst-convert binary'
		category: 'Gitocello'
		description: 'The gst-convert tool may be used to convert between Squeak and Gnu Smalltalk'
		type: #String>
	^GstConvertBinaryPath ifNil: [GstConvertBinaryPath := 'gst-convert']
! !

!GCGstConvertCommand class methodsFor: 'preferences' stamp: ''!
gstConvertBinaryPath: aString
	GstConvertBinaryPath := aString
! !

!GCGstConvertCommand class methodsFor: 'literals' stamp: ''!
gstNewline
	^'\\n'
! !

!GCGstConvertCommand class methodsFor: 'bidirectional-rules' stamp: ''!
methodNames
	^
	{'Date current' -> 'Date today'.
	'TimeStamp current' -> 'DateTime now'.
	'FileDirectory pathNameDelimiter' -> 'Directory pathSeparator'.
	'FileDirectory default' -> 'Directory working'.
	'(FileDirectory deleteFilePath: ``@arg2)' -> '(File remove: ``@arg2)'.
	'(FileDirectory localNameFor: ``@arg2)' 
	    -> '(FilePath stripPathFrom: ``@arg2)'.
	'(``@object recursiveDelete)' -> '(``@object all remove)'.
	'(``@object pathName)' -> '(``@object name)'.
	'(``@object containingDirectory)' -> '(``@object parent)'.
	'(``@object assureExistence)' -> '(``@object createDirectories)'.
	'(``@object fileExists: ``@arg2)' -> '(``@object / ``@arg2) exists'.
	'(FileDirectory on: ``@arg2)' -> '(``@arg2 asFile)'.
	'(``@object directoryNamed: ``@arg2)' -> '(``@object / ``@arg2)'.
	'(``@object readOnlyFileNamed: ``@arg2)' 
	    -> '(``@object / ``@arg2) readStream'.
	'(``@object forceNewFileNamed: ``@arg2)' 
	    -> '(``@object / ``@arg2) writeStream'.
	'(``@object allSubInstancesDo: ``@arg2)' 
	    -> '(``@object allSubinstancesDo: ``@arg2)'.
	'(``@object newFrom: ``@arg2)' -> '(``@object from: ``@arg2)'.
	'((``@object methodClass organization categoryOfElement: ``@object selector))' 
	    -> '(``@object methodCategory)'}
! !

!GCGstConvertCommand class methodsFor: 'preferences' stamp: ''!
runGstConvert
	<preference: 'Convert to Gnu Smalltalk'
		category: 'Gitocello'
		description: 'Convert  file-out to Gnu Smalltalk syntax using gst-convert'
		type: #Boolean>
	^RunGstConvert ifNil: [RunGstConvert := false]
! !

!GCGstConvertCommand class methodsFor: 'preferences' stamp: ''!
runGstConvert: aBoolean
	RunGstConvert := aBoolean
! !

!GCGstConvertCommand class methodsFor: 'squeak-to-gst-rules' stamp: ''!
squeakBidirectional
	"Find all bidirectional rules to include them in the conversion"
	^(self allMethodsIn: 'bidirectional-rules') inject: OrderedCollection new
	    into: 
		[:list :method | 
		(list asSet)
		    addAll: (self perform: method);
		    yourself]
! !

!GCGstConvertCommand class methodsFor: 'squeak-to-gst-rules' stamp: ''!
squeakClassNames
	^
	{'TimeStamp' -> 'DateTime'.
	'ProtoObject' -> 'nil'}
! !

!GCGstConvertCommand class methodsFor: 'squeak-to-gst-rules' stamp: ''!
squeakMethodNames
	^
	{'(``@object caseInsensitiveLessOrEqual: ``@arg2)' 
	    -> '(``@object <= ``@arg2)'.
	'(``@object isZero)' -> '(``@object = 0)'.
	'FileDirectory dot' -> '($. printString)'.
	'(``@object directoryExists: ``@arg2)' -> '(File path: ``@arg2) exists'.
	'(``@object union: ``@arg1)' 
	    -> '(``@object asSet addAll: ``@arg1; yourself)'.
	'FileStream st' -> '($s printString, $t printString)'.
	'(``@object beginsWith: ``@arg2)' -> '(``@object startsWith: ``@arg2)'.
	'(``@object ifNil: ``@arg1 ifNotNilDo: ``@arg2)' 
	    -> '(``@object ifNil: ``@arg1 ifNotNil: ``@arg2)'.
	'(``@object ifNotNilDo: ``@arg2 ifNil: ``@arg1)' 
	    -> '(``@object ifNotNil: ``@arg2 ifNil: ``@arg1)'.
	'(``@object ifNotNilDo: ``@arg2)' -> '(``@object ifNotNil: ``@arg2)'.
	'(``@object doIfNotNil: ``@arg2)' -> '(``@object ifNotNil: ``@arg2)'}
! !

!GCGstConvertCommand class methodsFor: 'literals' stamp: ''!
squeakNewline
	^'\\r'
! !

!GCGstConvertCommand class methodsFor: 'squeak-to-gst-rules' stamp: ''!
squeakStatements
	^
	{'(``@object and: ``@arg1 and: ``@arg2)' 
	    -> '((``@object and: ``@arg1) and: ``@arg2)'}
! !

!GCGstConvertCommand class methodsFor: 'bidirectional-rules' stamp: ''!
statements
	^
	{'(``@object ifNil: ``@arg ifNotNil: [ | `@t2 | `@.s2 ])' 
	    -> '(``@object ifNil: ``@arg ifNotNil: [ :foo || `@t2 | `@.s2 ])'.
	'(``@object ifNotNil: [ | `@t2 | `@.s2 ] ifNil: ``@arg)' 
	    -> '(``@object ifNotNil: [ :foo || `@t2 | `@.s2 ] ifNil: ``@arg)'.
	'(``@object ifNotNil: [ | `@t2 | `@.s2 ])' 
	    -> '(``@object ifNotNil: [ :foo || `@t2 | `@.s2 ])'}
! !

!GCGstConvertCommand class methodsFor: 'command-building' stamp: ''!
writeRules: someRules to: aStream
	someRules do: 
		[:each | 
		aStream
		    nextPutAll: '-r';
		    nextPut: $';
		    nextPutAll: (each printString copyWithout: $');
		    nextPut: $';
		    nextPut: Character space]
! !

Object subclass: #GCCallout
	instanceVariableNames: 'localPath'
	classVariableNames: ''
	poolDictionaries: 'CalloutDict GitBinaryPath'
	category: 'Gitocello-Core'!

!GCCallout class methodsFor: 'callout' stamp: ''!
apiLinuxLibc5Callout: aCommand
	<cdecl: long 'system' ( char* ) module: 'libc.so.5'>
	^self externalCallFailed
! !

!GCCallout class methodsFor: 'callout' stamp: ''!
apiLinuxLibc6Callout: aCommand
	<cdecl: long 'system' ( char* ) module: 'libc.so.6'>
	^self externalCallFailed
! !

!GCCallout class methodsFor: 'callout' stamp: ''!
apiMacOSXCallout: aCommand
	| errCode |
	<cdecl: long 'system' (char*) module: 'libSystem.dylib'>
	errCode := ExternalFunction getLastError.
	^errCode = 13 
	    ifTrue: 
		["Unable to find function address. dylibs are not searched"

		self 
		    error: 'Cannot call libSystem.dylib. 
			Please set "SqueakPluginsBuiltInOrLocalOnly" in your VM''s Info.plist to false']
	    ifFalse: [self externalCallFailed]
! !

!GCCallout class methodsFor: 'callout' stamp: ''!
apiWindowsCallout: aCommand
	<apicall: long 'system' ( char* ) module: 'msvcrt.dll'>
	^self externalCallFailed
! !

!GCCallout class methodsFor: 'callout' stamp: ''!
callout: aCommand
	^(Smalltalk classNamed: 'OSProcess') ifNil: 
		[(self calloutDict at: SmalltalkImage current platformName asLowercase
		    ifAbsent: [[:cmd | self error: 'Callouts not implemented for your platform!!']]) 
			value: aCommand]
	    ifNotNil: [:class | class waitForCommand: aCommand]
! !

!GCCallout class methodsFor: 'callout' stamp: ''!
calloutDict
	CalloutDict ifNil: 
		[CalloutDict := (Dictionary new)
			    at: 'unix' put: [:cmd | self linuxCallout: cmd];
			    at: 'win32' put: [:cmd | self windowsCallout: cmd];
			    at: 'mac os' put: [:cmd | self apiMacOSXCallout: cmd];
			    yourself].
	^CalloutDict
! !

!GCCallout class methodsFor: 'callout' stamp: ''!
linuxCallout: aCommand
	"Try with new libc6 and older libc5"
	[self apiLinuxLibc6Callout: aCommand] on: Error
	    do: [self apiLinuxLibc5Callout: aCommand]
! !

!GCCallout class methodsFor: 'callout' stamp: ''!
windowsCallout: aCommand
	"Escape newlines for Windows CmdLine"
	| escapedCommand |
	escapedCommand := (aCommand copyReplaceAll: Character cr asString with: ' ') 
		    copyReplaceAll: Character lf asString
		    with: ' '.
	self apiWindowsCallout: escapedCommand
! !

Object subclass: #GCGitWrapper
	instanceVariableNames: 'localPath'
	classVariableNames: ''
	poolDictionaries: 'CalloutDict GitBinaryPath'
	category: 'Gitocello-Core'!

!GCGitWrapper class methodsFor: 'preferences' stamp: ''!
gitBinaryPath
	<preference: 'Git binary path'
		category: 'Gitocello'
		description: 'Set the path of the git binary to be used for Gitocello'
		type: #String>
	^GitBinaryPath ifNil: ['git']
! !

!GCGitWrapper class methodsFor: 'preferences' stamp: ''!
gitBinaryPath: aString
	GitBinaryPath := aString
! !

!GCGitWrapper methodsFor: 'commands' stamp: ''!
add
	"Just add all"
	self gitCommand: 'add .'
! !

!GCGitWrapper methodsFor: 'commands' stamp: ''!
add: matchString
	"Add using matchString"
	self gitCommand: 'add ' , matchString
! !

!GCGitWrapper methodsFor: 'commands' stamp: ''!
commit: commitMsg
	"Commit changes to the local repository"
	self gitCommand: 'commit --allow-empty -m "' , commitMsg , '"'
! !

!GCGitWrapper methodsFor: 'commands' stamp: ''!
dirChangeCommand
	^SmalltalkImage current platformName asLowercase = 'win32' 
	    ifTrue: 
		['cd "' , self localPath , '" & ' , (self localPath copyUpTo: $:) , ': & ']
	    ifFalse: ['cd "' , self localPath , '";']
! !

!GCGitWrapper methodsFor: 'commands' stamp: ''!
gitCommand: aCommandString
	"Enter the local repository and run the 'git' command with the parameters"
	| command |
	command := self dirChangeCommand , ' ' , self class gitBinaryPath , ' ' 
		    , aCommandString.
	GCCallout callout: command
! !

!GCGitWrapper methodsFor: 'commands' stamp: ''!
init
	self gitCommand: 'init'
! !

!GCGitWrapper methodsFor: 'accessing' stamp: ''!
localPath
	"Answer the value of localPath"
	^localPath
! !

!GCGitWrapper methodsFor: 'accessing' stamp: ''!
localPath: anObject
	"Set the value of localPath"
	localPath := anObject
! !

!GCGitWrapper methodsFor: 'commands' stamp: ''!
origin: aRemoteUrl
	self gitCommand: 'remote rm origin'.
	self gitCommand: 'remote add origin ' , aRemoteUrl
! !

!GCGitWrapper methodsFor: 'commands' stamp: ''!
pull
	"Avoid merging altogether for now"
	self gitCommand: 'pull -s ours origin master'
! !

!GCGitWrapper methodsFor: 'commands' stamp: ''!
push
	self gitCommand: 'push origin master'
! !
GCRegistry initialize!
